@@ -1,11 +1,9 @@
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+For general Scribus (>=1.3.2) copyright and licensing information please refer
+to the COPYING file provided with the program. Following this notice may exist
+a copyright and/or license notice that predates the release of Scribus 1.3.2
+for which a new license (GPL+exception) is in place.
+*/
 #include "cmykfw.h"
 #include "cmykfw.moc"
 #include <qpainter.h>
@@ -15,32 +13,39 @@
 #include <qfileinfo.h>
 #include <qdir.h>
 #include <cstdlib>
-#ifdef _MSC_VER
- #if (_MSC_VER >= 1200)
-  #include "win-config.h"
- #endif
-#else
- #include "config.h"
-#endif
+
+#include "commonstrings.h"
+#include "sccombobox.h"
+#include "scconfig.h"
+#include "scpaths.h"
 #ifdef HAVE_CMS
 extern bool SoftProofing;
 extern bool Gamut;
 extern bool CMSuse;
 #endif
-extern QPixmap loadIcon(QString nam);
+#include "util.h"
 
-CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *Colors, QStringList Cust  )
+CMYKChoose::CMYKChoose( QWidget* parent, ScColor orig, QString name, ColorList *Colors, QStringList Cust  )
 		: QDialog( parent, "fw", true, 0 )
 {
-	CMYKmode = true;
+	if (orig.getColorModel () == colorModelCMYK)
+		CMYKmode = true;
+	else
+		CMYKmode = false;
 	dynamic = true;
 	Wsave = false;
 	EColors = Colors;
 	CurrSwatch.clear();
+	orig.checkGamut();
+	alertIcon = loadIcon("alert.png");
 	imageA = QPixmap(50,50);
-	imageA.fill(orig.getRGBColor());
+	imageA.fill(orig.getDisplayColor());
+	if (orig.isOutOfGamut())
+		paintAlert(alertIcon,imageA, 2, 2, false);
 	imageN = QPixmap(50,50);
-	imageN.fill(orig.getRGBColor());
+	imageN.fill(orig.getDisplayColor());
+	if (orig.isOutOfGamut())
+		paintAlert(alertIcon, imageN, 2, 2, false);
 	Farbe = orig;
 	QPixmap image0 = SliderPix(180);
 	QPixmap image1 = SliderPix(300);
@@ -53,6 +58,12 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	cmd = cm / 2.55;
 	cyd = cy / 2.55;
 	ckd = ck / 2.55;
+	int cr, cg, cb;
+	double crd, cgd, cbd;
+	orig.getRGB(&cr, &cg, &cb);
+	crd = cr / 2.55;
+	cgd = cg / 2.55;
+	cbd = cb / 2.55;
 	resize( 498, 306 );
 	setCaption( tr( "Edit Color" ) );
 	setIcon(loadIcon("AppIcon.png"));
@@ -77,20 +88,27 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	TextLabel3->setMinimumSize( QSize( 100, 22 ) );
 	Layout23->addWidget( TextLabel3 );
 
-	ComboBox1 = new QComboBox( true, this, "ComboBox1" );
-	ComboBox1->setEditable(false);
+	ComboBox1 = new ScComboBox( false, this, "ComboBox1" );
 	ComboBox1->insertItem( tr( "CMYK" ) );
 	ComboBox1->insertItem( tr( "RGB" ) );
 	ComboBox1->insertItem( tr( "Web Safe RGB" ) );
-	ComboBox1->setMinimumSize( QSize( 200, 22 ) );
+	if (!CMYKmode)
+		ComboBox1->setCurrentItem( 1 );
 	TextLabel3->setBuddy( ComboBox1 );
 	Layout23->addWidget( ComboBox1 );
-	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
-	Layout23->addItem( spacer );
 
-	/*    Separations = new QCheckBox( this, "Separations" );
-	    Separations->setText( tr( "Is Spot-Color" ) );
-	    Layout23->addWidget( Separations );    */
+	Separations = new QCheckBox( this, "Separations" );
+	Separations->setText( tr( "Is Spot Color" ) );
+	Separations->setChecked(orig.isSpotColor());
+	Layout23->addWidget( Separations );
+
+	Regist = new QCheckBox( this, "Regist" );
+	Regist->setText( tr( "Is Registration Color" ) );
+	Regist->setChecked(orig.isRegistrationColor());
+	Layout23->addWidget( Regist );
+
+	QSpacerItem* spacer = new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding );
+	Layout23->addItem( spacer );
 
 	Layout2 = new QGridLayout;
 	Layout2->setSpacing( 6 );
@@ -135,10 +153,10 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	Layout21->setSpacing( 20 );
 	Layout21->setMargin( 10 );
 
-	Cancel_2 = new QPushButton( tr( "&OK" ), this, "Cancel_2" );
+	Cancel_2 = new QPushButton( CommonStrings::tr_OK, this, "Cancel_2" );
 	Cancel_2->setDefault( true );
 	Layout21->addWidget( Cancel_2 );
-	Cancel = new QPushButton( tr( "&Cancel" ), this, "Cancel" );
+	Cancel = new QPushButton( CommonStrings::tr_Cancel, this, "Cancel" );
 	Layout21->addWidget( Cancel );
 	Layout23->addLayout( Layout21 );
 	CMYKFarbenLayout->addLayout( Layout23 );
@@ -150,13 +168,24 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	Frame4Layout->setSpacing( 6 );
 	Frame4Layout->setMargin( 0 );
 
-	Swatches = new QComboBox( true, Frame4, "ComboBox1" );
-	Swatches->setEditable(false);
+	Swatches = new ScComboBox( false, Frame4, "ComboBox1" );
 	Swatches->insertItem( tr( "HSV-Colormap" ) );
+/*
 	Swatches->insertItem("X11 RGB-Set");
 	Swatches->insertItem("X11 Grey-Set");
 	Swatches->insertItem("Gnome-Set");
 	Swatches->insertItem("SVG-Set");
+	Swatches->insertItem("OpenOffice.org-Set");
+*/
+	csm.findPaletteLocations();
+	csm.findPalettes();
+	QStringList allSets(csm.paletteNames());
+	for ( QStringList::Iterator it = allSets.begin(); it != allSets.end(); ++it )
+	{
+		Swatches->insertItem((*it));
+	}
+	customSetStartIndex=Swatches->count();
+
 	if (Cust.count() != 0)
 	{
 		QStringList realEx;
@@ -329,6 +358,8 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	BlackSL->setValue(qRound(ckd));
 	BlackComp = ck;
 	Frame4Layout->addLayout( Layout2x );
+	QSpacerItem* spacer2 = new QSpacerItem( 2, 2, QSizePolicy::Minimum, QSizePolicy::Expanding );
+	Frame4Layout->addItem( spacer2 );
 	CMYKFarbenLayout->addWidget( Frame4 );
 	int h, s, v;
 	orig.getRGBColor().hsv(&h, &s, &v);
@@ -338,7 +369,10 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	Farbname->selectAll();
 	Farbname->setFocus();
 	TabStack->raiseWidget(0);
+	setFixedSize(minimumSizeHint());
 	// signals and slots connections
+	QToolTip::add( Regist, "<qt>" + tr( "Choosing this will enable printing this on all plates. Registration colors are used for printer marks such as crop marks, registration marks and the like. These are not typically used in the layout itself." ) + "</qt>");
+	QToolTip::add( Separations, "<qt>" + tr( "Choosing this will make this color a spot color, thus creating another spot when creating plates or separations. This is used most often when a logo or other color needs exact representation or cannot be replicated with CMYK inks. Metallic and fluorescent inks are good examples which cannot be easily replicated with CMYK inks." ) + "</qt>");
 	connect( Cancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
 	connect( Cancel_2, SIGNAL( clicked() ), this, SLOT( Verlassen() ) );
 	connect( CyanSp, SIGNAL( valueChanged(int) ), CyanSL, SLOT( setValue(int) ) );
@@ -357,6 +391,11 @@ CMYKChoose::CMYKChoose( QWidget* parent, CMYKColor orig, QString name, CListe *C
 	connect( ComboBox1, SIGNAL(activated(const QString&)), this, SLOT(SelModel(const QString&)));
 	connect( Swatches, SIGNAL(activated(int)), this, SLOT(SelSwatch(int)));
 	connect(ColorSwatch, SIGNAL(highlighted(int)), this, SLOT(SelFromSwatch(int)));
+	connect(Separations, SIGNAL(clicked()), this, SLOT(setSpot()));
+	connect(Regist, SIGNAL(clicked()), this, SLOT(setRegist()));
+	layout()->activate();
+	if (!CMYKmode)
+		SelModel ( tr( "RGB" ));
 }
 
 void CMYKChoose::mouseReleaseEvent(QMouseEvent *m)
@@ -411,61 +450,80 @@ QPixmap CMYKChoose::SliderPix(int farbe)
 	p.begin(&image0);
 	p.setPen(NoPen);
 	int r, g, b, c, m, y, k;
-	Farbe.getCMYK(&c, &m, &y, &k);
-#ifdef HAVE_CMS
-	if ((Gamut) && (CMSuse))
-	{
-		QColor tmp3 = CMYK2RGB(c, m, y, k);
-		tmp3.rgb(&r, &g, &b);
-	}
-	else
-		Farbe.getRGBColor().rgb(&r, &g, &b);
-#else
-	Farbe.getRGBColor().rgb(&r, &g, &b);
-#endif
 	QColor tmp;
 	for (int x = 0; x < 255; x += 5)
 	{
 		if (CMYKmode)
 		{
+			Farbe.getCMYK(&c, &m, &y, &k);
 			if (dynamic)
 			{
 				switch (farbe)
 				{
 				case 180:
-					tmp = CMYK2RGB(x, m, y, k);
+					tmp = ScColor(x, m, y, k).getDisplayColorGC();
 					break;
 				case 300:
-					tmp = CMYK2RGB(c, x, y, k);
+					tmp = ScColor(c, x, y, k).getDisplayColorGC();
 					break;
 				case 60:
-					tmp = CMYK2RGB(c, m, x, k);
+					tmp = ScColor(c, m, x, k).getDisplayColorGC();
 					break;
 				}
 				p.setBrush(tmp);
 			}
 			else
-				p.setBrush(QColor(farbe, x, 255, QColor::Hsv));
+			{
+				switch (farbe)
+				{
+				case 180:
+					tmp = ScColor(x, 0, 0, 0).getDisplayColorGC();
+					break;
+				case 300:
+				        tmp = ScColor(0, x, 0, 0).getDisplayColorGC();
+					break;
+				case 60:
+					tmp = ScColor(0, 0, x, 0).getDisplayColorGC();
+					break;
+				}
+				p.setBrush(tmp);
+			}
 		}
 		else
 		{
+			Farbe.getRGB(&r, &g, &b);
 			if (dynamic)
 			{
 				switch (farbe)
 				{
 				case 0:
-					p.setBrush(QColor(x, g, b, QColor::Rgb));
+					tmp = ScColor(x, g, b).getDisplayColorGC();
 					break;
 				case 120:
-					p.setBrush(QColor(r, x, b, QColor::Rgb));
+					tmp = ScColor(r, x, b).getDisplayColorGC();
 					break;
 				case 240:
-					p.setBrush(QColor(r, g, x, QColor::Rgb));
+					tmp = ScColor(r, g, x).getDisplayColorGC();
 					break;
 				}
+				p.setBrush(tmp);
 			}
 			else
-				p.setBrush(QColor(farbe, 255, x, QColor::Hsv));
+			{
+				switch (farbe)
+				{
+				case 0:
+					tmp = ScColor(x, 0, 0).getDisplayColorGC();
+					break;
+				case 120:
+				        tmp = ScColor(0, x, 0).getDisplayColorGC();
+					break;
+				case 240:
+					tmp = ScColor(0, 0, x).getDisplayColorGC();
+					break;
+				}
+				p.setBrush(tmp);
+			}
 		}
 		p.drawRect(x, 0, 5, 10);
 	}
@@ -485,9 +543,9 @@ QPixmap CMYKChoose::SliderBlack()
 	for (int x = 0; x < 255; x += 5)
 	{
 		if (dynamic)
-			p.setBrush(CMYK2RGB(c, m, y, x));
+			p.setBrush(ScColor(c, m, y, x).getDisplayColorGC());
 		else
-			p.setBrush(QColor(val, val, val, QColor::Rgb));
+			p.setBrush(ScColor(0, 0, 0, x).getDisplayColorGC());
 		p.drawRect(x, 0, 5, 10);
 		val -= 5;
 	}
@@ -504,26 +562,42 @@ void CMYKChoose::SelSwatch(int n)
 		bool cus = false;
 		CurrSwatch.clear();
 		QString Cpfad = QDir::convertSeparators(QDir::homeDirPath()+"/.scribus/"+Swatches->currentText());
-		QString pfadC = LIBDIR;
-		QString pfadC2 = pfadC + "rgbscribus.txt";
+		QString pfadC = ScPaths::instance().libDir()+"swatches/";
+		QString pfadC2 = pfadC + "Scribus_X11.txt";
 		switch (n)
 		{
+			/*
 		case 1:
-			pfadC2 = pfadC + "rgbscribus.txt";
+			pfadC2 = pfadC + "Scribus_X11.txt";
 			break;
 		case 2:
-			pfadC2 = pfadC + "rgbscribusgreys.txt";
+			pfadC2 = pfadC + "Scribus_X11Grey.txt";
 			break;
 		case 3:
-			pfadC2 = pfadC + "rgbscribusgnome.txt";
+			pfadC2 = pfadC + "Scribus_Gnome.txt";
 			break;
 		case 4:
-			pfadC2 = pfadC + "rgbsvg.txt";
+			pfadC2 = pfadC + "Scribus_SVG.txt";
 			break;
-		default:
-			pfadC2 = Cpfad;
+		case 5:
+			pfadC2 = pfadC + "Scribus_OpenOffice.txt";
 			cus = true;
 			break;
+			*/
+		default:
+			if (n<customSetStartIndex)
+			{
+				QString listText=Swatches->text(n);
+				if (listText=="Scribus OpenOffice")
+					cus=true;
+				pfadC2 = csm.paletteFileFromName(listText);
+			}
+			else
+			{
+				pfadC2 = Cpfad;
+				cus = true;
+			}
+			break;
 		}
 		if (n != 0)
 		{
@@ -536,8 +610,10 @@ void CMYKChoose::SelSwatch(int n)
 				ColorEn = tsC.readLine();
 				while (!tsC.atEnd())
 				{
-					CMYKColor tmp;
+					ScColor tmp;
 					ColorEn = tsC.readLine();
+					if (ColorEn.length()>0 && ColorEn[0]==QChar('#'))
+						continue;
 					QTextStream CoE(&ColorEn, IO_ReadOnly);
 					CoE >> Rval;
 					CoE >> Gval;
@@ -553,28 +629,44 @@ void CMYKChoose::SelSwatch(int n)
 						Cname = CoE.read().stripWhiteSpace();
 						tmp.setColorRGB(Rval, Gval, Bval);
 					}
+
+					if ((n<customSetStartIndex) && (Cname.length()==0))
+					{
+						if (!cus)
+							Cname=QString("#%1%2%3").arg(Rval,2,16).arg(Gval,2,16).arg(Bval,2,16).upper();
+						else
+							Cname=QString("#%1%2%3%4").arg(Rval,2,16).arg(Gval,2,16).arg(Bval,2,16).arg(Kval,2,16).upper();
+						Cname.replace(" ","0");
+					}
+					if (CurrSwatch.contains(Cname))
+					{
+						if (tmp==CurrSwatch[Cname])
+							continue;
+						Cname=QString("%1%2").arg(Cname).arg(CurrSwatch.count());
+					}
+
 					CurrSwatch.insert(Cname, tmp);
 				}
 				fiC.close();
 			}
 			else
 			{
-				CurrSwatch.insert("White", CMYKColor(0, 0, 0, 0));
-				CurrSwatch.insert("Black", CMYKColor(0, 0, 0, 255));
-				CurrSwatch.insert("Blue", CMYKColor(255, 255, 0, 0));
-				CurrSwatch.insert("Cyan", CMYKColor(255, 0, 0, 0));
-				CurrSwatch.insert("Green", CMYKColor(255, 0, 255, 0));
-				CurrSwatch.insert("Red", CMYKColor(0, 255, 255, 0));
-				CurrSwatch.insert("Yellow", CMYKColor(0, 0, 255, 0));
-				CurrSwatch.insert("Magenta", CMYKColor(0, 255, 0, 0));
+				CurrSwatch.insert("White", ScColor(0, 0, 0, 0));
+				CurrSwatch.insert("Black", ScColor(0, 0, 0, 255));
+				CurrSwatch.insert("Blue", ScColor(255, 255, 0, 0));
+				CurrSwatch.insert("Cyan", ScColor(255, 0, 0, 0));
+				CurrSwatch.insert("Green", ScColor(255, 0, 255, 0));
+				CurrSwatch.insert("Red", ScColor(0, 255, 255, 0));
+				CurrSwatch.insert("Yellow", ScColor(0, 0, 255, 0));
+				CurrSwatch.insert("Magenta", ScColor(0, 255, 0, 0));
 			}
 		}
 		ColorSwatch->clear();
-		CListe::Iterator it;
+		ColorList::Iterator it;
 		QPixmap pm = QPixmap(30, 15);
 		for (it = CurrSwatch.begin(); it != CurrSwatch.end(); ++it)
 		{
-			pm.fill(CurrSwatch[it.key()].getRGBColor());
+			pm.fill(CurrSwatch[it.key()].getDisplayColor());
 			ColorSwatch->insertItem(pm, it.key());
 		}
 		ColorSwatch->setSelected(ColorSwatch->currentItem(), false);
@@ -582,6 +674,30 @@ void CMYKChoose::SelSwatch(int n)
 	}
 }
 
+void CMYKChoose::setRegist()
+{
+	disconnect( ComboBox1, SIGNAL(activated(const QString&)), this, SLOT(SelModel(const QString&)));
+	if (Regist->isChecked())
+	{
+		ComboBox1->setCurrentItem( 0 );
+		Separations->setChecked(false);
+		SelModel( tr("CMYK"));
+	}
+	connect( ComboBox1, SIGNAL(activated(const QString&)), this, SLOT(SelModel(const QString&)));
+}
+
+void CMYKChoose::setSpot()
+{
+	disconnect( ComboBox1, SIGNAL(activated(const QString&)), this, SLOT(SelModel(const QString&)));
+	if (Separations->isChecked())
+	{
+		ComboBox1->setCurrentItem( 0 );
+		Regist->setChecked(false);
+		SelModel( tr("CMYK"));
+	}
+	connect( ComboBox1, SIGNAL(activated(const QString&)), this, SLOT(SelModel(const QString&)));
+}
+
 void CMYKChoose::SelModel(const QString& mod)
 {
 	disconnect( CyanSp, SIGNAL( valueChanged(int) ), CyanSL, SLOT( setValue(int) ) );
@@ -628,10 +744,15 @@ void CMYKChoose::SelModel(const QString& mod)
 		MagentaP->setPixmap(SliderPix(300));
 		YellowP->setPixmap(SliderPix(60));
 		BlackP->setPixmap(SliderBlack());
+/*		BlackP->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+		BlackSL->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+		BlackSp->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+		BlackT->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum)); */
 		BlackP->show();
 		BlackSL->show();
 		BlackSp->show();
 		BlackT->show();
+		Farbe.setColorModel(colorModelCMYK);
 		setValues();
 	}
 	else
@@ -666,6 +787,11 @@ void CMYKChoose::SelModel(const QString& mod)
 		CyanP->setPixmap(SliderPix(0));
 		MagentaP->setPixmap(SliderPix(120));
 		YellowP->setPixmap(SliderPix(240));
+		Layout2x->setResizeMode(QLayout::Fixed);
+/*		BlackP->setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored));
+		BlackSL->setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored));
+		BlackSp->setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored));
+		BlackT->setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored)); */
 		BlackP->hide();
 		BlackSL->hide();
 		BlackSp->hide();
@@ -683,8 +809,14 @@ void CMYKChoose::SelModel(const QString& mod)
 			MagentaSp->setLineStep(51);
 			YellowSp->setLineStep(51);
 		}
+		Farbe.setColorModel(colorModelRGB);
 		setValues();
 	}
+	imageN.fill(Farbe.getDisplayColor());
+	Farbe.checkGamut();
+	if (Farbe.isOutOfGamut())
+		paintAlert(alertIcon, imageN, 2, 2, false);
+	NewC->setPixmap( imageN );
 	connect( CyanSp, SIGNAL( valueChanged(int) ), CyanSL, SLOT( setValue(int) ) );
 	connect( MagentaSp, SIGNAL( valueChanged(int) ), MagentaSL, SLOT( setValue(int) ) );
 	connect( YellowSp, SIGNAL( valueChanged(int) ), YellowSL, SLOT( setValue(int) ) );
@@ -704,10 +836,10 @@ void CMYKChoose::setColor()
 	int c, m, y, k;
 	if (CMYKmode)
 	{
-		c = qRound(CyanSp->value() * 255 / 100);
-		m = qRound(MagentaSp->value() * 255 / 100);
-		y = qRound(YellowSp->value() * 255 / 100);
-		k = qRound(BlackSp->value() * 255 / 100);
+		c = qRound(CyanSp->value() * 2.55);
+		m = qRound(MagentaSp->value() * 2.55);
+		y = qRound(YellowSp->value() * 2.55);
+		k = qRound(BlackSp->value() * 2.55);
 	}
 	else
 	{
@@ -732,9 +864,10 @@ void CMYKChoose::setColor()
 		YellowSL->setValue(y);
 		blockSignals(false);
 	}
-	CMYKColor tmp = CMYKColor(c, m, y, k);
+	ScColor tmp;
 	if (CMYKmode)
 	{
+		tmp.setColor(c, m, y, k);
 		if (dynamic)
 		{
 			CyanP->setPixmap(SliderPix(180));
@@ -745,10 +878,9 @@ void CMYKChoose::setColor()
 	}
 	else
 	{
+		 tmp.setColorRGB(c, m, y);
 		QColor tmp2 = QColor(c, m, y, QColor::Rgb);
 		tmp2.hsv(&h, &s, &v);
-		tmp.fromQColor(tmp2);
-		tmp.RecalcRGB();
 		BlackComp = 255 - v;
 		if (dynamic)
 		{
@@ -757,24 +889,11 @@ void CMYKChoose::setColor()
 			YellowP->setPixmap(SliderPix(240));
 		}
 	}
-#ifdef HAVE_CMS
-	if ((Gamut) && (CMSuse))
-	{
-		int cc, cm, cy, ck;
-		tmp.getCMYK(&cc, &cm, &cy, &ck);
-		QColor tmp3 = CMYK2RGB(cc, cm, cy, ck);
-		imageN.fill(tmp3);
-		tmp3.hsv(&h, &s, &v);
-	}
-	else
-	{
-		imageN.fill(tmp.getRGBColor());
-		tmp.getRGBColor().hsv(&h, &s, &v);
-	}
-#else
-	imageN.fill(tmp.getRGBColor());
+	imageN.fill(tmp.getDisplayColor());
+	tmp.checkGamut();
+	if (tmp.isOutOfGamut())
+		paintAlert(alertIcon, imageN, 2, 2, false);
 	tmp.getRGBColor().hsv(&h, &s, &v);
-#endif
 	NewC->setPixmap( imageN );
 	Farbe = tmp;
 	ColorMap->drawPalette(v);
@@ -786,30 +905,14 @@ void CMYKChoose::setColor2(int h, int s, bool ende)
 	QColor tm = QColor(QMAX(QMIN(359,h),0), QMAX(QMIN(255,255-s),0), 255-BlackComp, QColor::Hsv);
 	int r, g, b;
 	tm.rgb(&r, &g, &b);
-	CMYKColor tmp = CMYKColor();
+	ScColor tmp;
+	tmp.fromQColor(tm);
 	if (CMYKmode)
-	{
-		int k = qRound(BlackSp->value() * 255 / 100);
-		int c = QMAX(255 - r - k, 0);
-		int m = QMAX(255 - g - k, 0);
-		int y = QMAX(255 - b - k, 0);
-		tmp.setColor(c, m, y, k);
-	}
-	else
-		tmp.fromQColor(tm);
-#ifdef HAVE_CMS
-	if ((Gamut) && (CMSuse))
-	{
-		int cc, cm, cy, ck;
-		tmp.getCMYK(&cc, &cm, &cy, &ck);
-		QColor tmp3 = CMYK2RGB(cc, cm, cy, ck);
-		imageN.fill(tmp3);
-	}
-	else
-		imageN.fill(tmp.getRGBColor());
-#else
-	imageN.fill(tmp.getRGBColor());
-#endif
+		tmp.setColorModel(colorModelCMYK);
+	tmp.checkGamut();
+	imageN.fill(tmp.getDisplayColor());
+	if (tmp.isOutOfGamut())
+		paintAlert(alertIcon, imageN, 2, 2, false);
 	NewC->setPixmap( imageN );
 	Farbe = tmp;
 	if (ende)
@@ -818,20 +921,13 @@ void CMYKChoose::setColor2(int h, int s, bool ende)
 
 void CMYKChoose::SelFromSwatch(int c)
 {
-	CMYKColor tmp = CurrSwatch[ColorSwatch->text(c)];
-#ifdef HAVE_CMS
-	if ((Gamut) && (CMSuse))
-	{
-		int cc, cm, cy, ck;
-		tmp.getCMYK(&cc, &cm, &cy, &ck);
-		QColor tmp3 = CMYK2RGB(cc, cm, cy, ck);
-		imageN.fill(tmp3);
-	}
-	else
-		imageN.fill(tmp.getRGBColor());
-#else
-	imageN.fill(tmp.getRGBColor());
-#endif
+	ScColor tmp = CurrSwatch[ColorSwatch->text(c)];
+	if (CMYKmode)
+		tmp.setColorModel(colorModelCMYK);
+	tmp.checkGamut();
+	imageN.fill(tmp.getDisplayColor());
+	if (tmp.isOutOfGamut())
+		paintAlert(alertIcon, imageN, 2, 2, false);
 	NewC->setPixmap( imageN );
 	Farbe = tmp;
 	setValues();
@@ -861,11 +957,8 @@ void CMYKChoose::setValues()
 	}
 	else
 	{
-		int cc, cm, cy, ck;
-		Farbe.getCMYK(&cc, &cm, &cy, &ck);
-		QColor tmp = CMYK2RGB(cc, cm, cy, ck);
 		int r, g, b;
-		tmp.rgb(&r, &g, &b);
+		Farbe.getRGB(&r, &g, &b);
 		CyanSp->setValue(static_cast<double>(r));
 		CyanSL->setValue(r);
 		MagentaSp->setValue(static_cast<double>(g));
@@ -873,7 +966,7 @@ void CMYKChoose::setValues()
 		YellowSp->setValue(static_cast<double>(b));
 		YellowSL->setValue(b);
 		int h, s, v;
-		tmp.hsv(&h, &s, &v);
+		Farbe.getRGBColor().hsv(&h, &s, &v);
 		BlackComp = 255 - v;
 		if (dynamic)
 		{
@@ -886,15 +979,16 @@ void CMYKChoose::setValues()
 
 QColor CMYKChoose::CMYK2RGB(int c, int m, int y, int k)
 {
+	/* this code should no longer be used */
 	return QColor(255-QMIN(255, c+k), 255-QMIN(255,m+k), 255-QMIN(255,y+k));
 }
 
 void CMYKChoose::Verlassen()
 {
 	// if condition 10/21/2004 pv #1191 - just be sure that user cannot create "None" color
-	if (Farbname->text() == "None" || Farbname->text() == tr("None"))
+	if (Farbname->text() == CommonStrings::None || Farbname->text() == CommonStrings::NoneColor)
 	{
-		QMessageBox::information(this, tr("Warning"), tr("You cannot create a color named \"%1\".\nIt's a reserved name for transparent color").arg(Farbname->text()), 0);
+		QMessageBox::information(this, CommonStrings::trWarning, tr("You cannot create a color named \"%1\".\nIt is a reserved name for transparent color").arg(Farbname->text()), 0);
 		Farbname->setFocus();
 		Farbname->selectAll();
 		return;
@@ -903,7 +997,7 @@ void CMYKChoose::Verlassen()
 	{
 		if (EColors->contains(Farbname->text()))
 		{
-			QMessageBox::information(this, tr("Warning"), tr("Name of the Color is not unique"), tr("OK"), 0, 0, 0, QMessageBox::Ok);
+			QMessageBox::information(this, CommonStrings::trWarning, tr("Name of the color is not unique"), CommonStrings::tr_OK, 0, 0, 0, QMessageBox::Ok);
 			Farbname->selectAll();
 			Farbname->setFocus();
 			return;
